def get_database_name(user, frequence, underlying, algo, starting_date, running_date, db_path_suffix):
    return  user + '_' + frequence + '_' + underlying + '_' + algo + '_'+ starting_date.strftime(
        '%d_%b_%Y') + '_' + running_date.strftime('%d_%b_%Y') + db_path_suffix

def get_cointegration_database_name(user, frequence, underlying_x, underlying_y, starting_date, running_date, db_path_suffix):
    return  user + '_' + frequence + '_' + underlying_x + '_' + underlying_y + '_'+ starting_date.strftime(
        '%d_%b_%Y') + '_' + running_date.strftime('%d_%b_%Y') + db_path_suffix

def get_daily_database_name(user, frequence, underlying, algo, starting_date, running_date, delay, db_path_suffix):
    return  user + '_' + frequence + '_' + underlying + '_' + algo + '_'+ starting_date.strftime(
        '%d_%b_%Y') + '_' + running_date.strftime('%d_%b_%Y')+ '_' + str(delay) + db_path_suffix


__author__ = "hugo.inzirillo"

from itertools import product
from typing import List

from bifrost import NapoleonQuoteServiceConnector
from pandas import Timestamp, Timedelta
from datetime import time
from napoleontoolbox.models.filter import EodQuoteFilter
from napoleontoolbox.models.product import EodQuoteProductCodeBuilder
from napoleontoolbox.models.quote import EodQuote


class NapoleonQuoteService(object):
    def __init__(self):
        self.__quote_connector = NapoleonQuoteServiceConnector()

    @property
    def position_connector(self):
        return self.__quote_connector

    def filter(self, filter: EodQuoteFilter) -> List[EodQuote]:
        return [EodQuote().from_dict(quote) for quote in self.position_connector.filter(data=filter)]

    def __check_filter(self, filter: EodQuoteFilter):
        start_date = Timestamp(filter.minDate)
        end_date = Timestamp(filter.maxDate)
        if end_date - start_date < 1:
            return True
        else:
            raise RuntimeError("Cannot request more than 1 year by request")



#!/usr/bin/env python
# coding: utf-8


def generate_augmento_simple(transform_featuress, los, magnitudes, nb_selecteds, leading_correlationss):
    parameters = []
    for transform_features in transform_featuress:
        for lo in los:
            for magnitude in magnitudes:
                for nb_selected in nb_selecteds:
                    for leading_correlations in leading_correlationss:
                        parameters.append({
                            'transform_features':transform_features,
                            'lo':lo,
                            'magnitude':magnitude,
                            'nb_selected':nb_selected,
                            'leading_correlations':leading_correlations
                        })
    return parameters

def generate_simple_baseline(thresholds, signal_action_types, mixing_types):
    parameters = []
    for threshold in thresholds:
        for signal_action_type in signal_action_types:
            for mixing_type in mixing_types:
                parameters.append({
                    'threshold': threshold,
                    'signal_action_type': signal_action_type,
                    'mixing_type': mixing_type,
                })
    return parameters

def generate_anova_like(signal_action_types, mixing_types):
    parameters = []
    for signal_action_type in signal_action_types:
        for mixing_type in mixing_types:
            parameters.append({
                'signal_action_type': signal_action_type,
                'mixing_type': mixing_type,
            })
    return parameters




def generate_lead_lag(lookback_windows, lead_lags, contravariants, ):
    parameters = []
    for contravariant in contravariants:
        for lookback_window in lookback_windows:
            for lead in lead_lags:
                if lead < lookback_window:
                    parameters.append({
                        'lead':lead,
                        'lookback_window':lookback_window,
                        'contravariant':contravariant
                    })
    return parameters

def generate_dd_threshold_lo(lookback_windows, up_thresholds, lags, contravariants):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for up_threshold in up_thresholds:
                for lag in lags:
                    if lag <= int(lookback_window/2.):
                        parameters.append({
                            'lookback_window':lookback_window,
                            'contravariant':contravariant,
                            'up_threshold':up_threshold,
                            'lag':lag
                        })
    return parameters

def generate_dd_threshold_ls(lookback_windows, low_thresholds, up_thresholds, lags, contravariants):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for low_threshold in low_thresholds:
                for up_threshold in up_thresholds:
                    for lag in lags:
                        if lag <= int(lookback_window/2.):
                            parameters.append({
                                'lookback_window':lookback_window,
                                'contravariant':contravariant,
                                'low_threshold':low_threshold,
                                'up_threshold':up_threshold,
                                'lag':lag
                            })
    return parameters

def generate_lookback_only(lookback_windows):
    parameters = []
    for lookback_window in lookback_windows:
                parameters.append({
                    'lookback_window':lookback_window
                })
    return parameters


def generate_lo_slope_ma(lookback_windows, pente_windows, contravariants,):
    parameters = []
    for lookback_window in lookback_windows:
        for pente_window in pente_windows:
            for contravariant in contravariants:
                if pente_window <= lookback_window:
                    parameters.append({
                        'lookback_window':lookback_window,
                        'pente_window':pente_window,
                        'contravariant':contravariant
                    })
    return parameters


# def generate_fourier_wvt_decomposition(lookback_windows,contravariants,n_harms,fft_filter_types,dec_levels,thresholds,obs_covs):
#     parameters = []
#     for lookback_window in lookback_windows:
#         for contravariant in contravariants:
#             for fft_filter_type in fft_filter_types:
#                 for n_harm in n_harms:
#                     for dec_level in dec_levels:
#                         for threshold in thresholds:
#                             for observation_covariance in obs_covs:
#                                 if n_harm < lookback_window:
#                                     parameters.append({
#                                         'lookback_window':lookback_window,
#                                         'contravariant':contravariant,
#                                         'fft_filter_type':fft_filter_type,
#                                         'n_harm':n_harm,
#                                         'dec_level':dec_level,
#                                         'threshold':threshold,
#                                         'observation_covariance':observation_covariance
#                                     })
#     return parameters

def generate_fourier_wvt_decomposition(lookback_windows,contravariants,n_harms,fft_filter_types,wavelet_filters,dec_levels,optimize_filtering_states, thresholds, soft_filtering_states):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for fft_filter_type in fft_filter_types:
                for n_harm in n_harms:
                    for dec_level in dec_levels:
                        for filter_name in wavelet_filters:
                            for threshold in thresholds:
                                for optimize_filtering in optimize_filtering_states:
                                    for soft_filtering in soft_filtering_states:
                                        if n_harm < lookback_window:
                                            if optimize_filtering:
                                                parameters.append({
                                                    'lookback_window':lookback_window,
                                                    'contravariant':contravariant,
                                                    'fft_filter_type':fft_filter_type,
                                                    'n_harm':n_harm,
                                                    'filter_name':filter_name,
                                                    'dec_level':dec_level,
                                                    'threshold':threshold,
                                                    'optimize_filtering':optimize_filtering,
                                                    'soft_filtering':soft_filtering
                                                })
                                            else:
                                                parameters.append({
                                                    'lookback_window': lookback_window,
                                                    'contravariant': contravariant,
                                                    'fft_filter_type': fft_filter_type,
                                                    'n_harm': n_harm,
                                                    'filter_name': filter_name,
                                                    'dec_level': dec_level,
                                                    'optimize_filtering': optimize_filtering,
                                                    'soft_filtering': soft_filtering
                                                })

    return parameters

def generate_fourier_wvt_ortho_decomposition(lookback_windows,n_harms,fft_filter_types,wavelet_filters,dec_levels,coef_factors, independent_selection_states):
    parameters = []
    for lookback_window in lookback_windows:
        for fft_filter_type in fft_filter_types:
            for n_harm in n_harms:
                for dec_level in dec_levels:
                    for filter_name in wavelet_filters:
                        for coeff_index_factor in coef_factors:
                            for independent_selection in independent_selection_states:
                                if n_harm < lookback_window:
                                    parameters.append({
                                        'lookback_window':lookback_window,
                                        'fft_filter_type':fft_filter_type,
                                        'n_harm':n_harm,
                                        'filter_name':filter_name,
                                        'dec_level':dec_level,
                                        'coeff_index_factor':coeff_index_factor,
                                        'independent_selection':independent_selection
                                    })
    return parameters

# def generate_kalman(lookback_windows,contravariants,n_harms,fft_filter_types,thresholds,obs_covs):
#     parameters = []
#     for lookback_window in lookback_windows:
#         for contravariant in contravariants:
#             for fft_filter_type in fft_filter_types:
#                 for n_harm in n_harms:
#                     for threshold in thresholds:
#                         for observation_covariance in obs_covs:
#                             if n_harm < lookback_window:
#                                 parameters.append({
#                                     'lookback_window':lookback_window,
#                                     'contravariant':contravariant,
#                                     'fft_filter_type':fft_filter_type,
#                                     'n_harm':n_harm,
#                                     'threshold':threshold,
#                                     'observation_covariance':observation_covariance
#                                 })
#     return parameters


def generate_kalman(lookback_windows,contravariants,n_harms,fft_filter_types,obs_covs):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for fft_filter_type in fft_filter_types:
                for n_harm in n_harms:
                    for observation_covariance in obs_covs:
                        if n_harm < lookback_window:
                            parameters.append({
                                'lookback_window':lookback_window,
                                'contravariant':contravariant,
                                'fft_filter_type':fft_filter_type,
                                'n_harm':n_harm,
                                'observation_covariance':observation_covariance
                            })
    return parameters



def generate_fourier_hht(lookback_windows,contravariants,n_harms,fft_filter_types, threshold_1_values, threshold_2_values, alpha_values, n_imfs_values):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for fft_filter_type in fft_filter_types:
                for n_harm in n_harms:
                    for threshold_1 in threshold_1_values:
                        for threshold_2 in threshold_2_values:
                            for alpha in alpha_values:
                                for n_imfs in n_imfs_values:
                                    if n_harm < lookback_window:
                                        parameters.append({
                                            'lookback_window': lookback_window,
                                            'contravariant': contravariant,
                                            'fft_filter_type': fft_filter_type,
                                            'n_harm': n_harm,
                                            'threshold_1':threshold_1,
                                            'threshold_2':threshold_2,
                                            'alpha':alpha,
                                            'n_imfs':n_imfs
                                        })
    return parameters


def generate_fourier_decomposition(lookback_windows,contravariants,n_harms,fft_filter_types,):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for fft_filter_type in fft_filter_types:
                for n_harm in n_harms:
                    if n_harm<lookback_window:
                        parameters.append({
                            'lookback_window':lookback_window,
                            'fft_filter_type': fft_filter_type,
                            'contravariant':contravariant,
                            'n_harm':n_harm
                        })
    return parameters

def generate_rooftingFilter(lookback_windows, periods, n_harms, fft_filter_types, contravariants):
    parameters = []
    for lookback_window in lookback_windows:
        for period in periods:
            for n_harm in n_harms:
                for fft_filter_type in fft_filter_types:
                    for contravariant in contravariants:
                        if period < lookback_window:
                            parameters.append({
                                'lookback_window': lookback_window,
                                'period':period,
                                'n_harm':n_harm,
                                'fft_filter_type':fft_filter_type,
                                'contravariant':contravariant
                            })
    return parameters


def generate_lookback_contravariant(lookback_windows, contravariants):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            parameters.append({
                'lookback_window':lookback_window,
                'contravariant':contravariant
            })
    return parameters

def generate_alpha_5(lookback_windows, lags, contravariants):
    parameters = []
    for lookback_window in lookback_windows:
        for lag in lags:
            for contravariant in contravariants:
                if lag < int(lookback_window/2):
                    parameters.append({
                        'lookback_window':lookback_window,
                        'lag':lag,
                        'contravariant':contravariant
                    })
    return parameters

def generate_alpha_8(lookback_windows, contravariants, lags):
    parameters = []
    for lookback_window in lookback_windows:
        for lag in lags:
            for contravariant in contravariants:
                if lag <= lookback_window/4:
                    parameters.append({
                        'lookback_window':lookback_window,
                        'lag' : lag,
                        'contravariant':contravariant
                    })
    return parameters


def generate_alpha_16(lookback_windows, contravariants, lags):
    parameters = []
    for lookback_window in lookback_windows:
        for lag in lags:
            for contravariant in contravariants:
                if lag <= lookback_window/4 and lag >= 2:
                    parameters.append({
                        'lookback_window':lookback_window,
                        'lag' : lag,
                        'contravariant':contravariant
                    })
    return parameters

def generate_alpha_16_lo(lookback_windows, contravariants, lags, up_thresholds):
    parameters = []
    for lookback_window in lookback_windows:
        for up_threshold in up_thresholds:
            for lag in lags:
                for contravariant in contravariants:
                    if lag <= lookback_window/4 and lag >= 2:
                        parameters.append({
                            'lookback_window':lookback_window,
                            'lag' : lag,
                            'up_threshold': up_threshold,
                            'contravariant':contravariant
                        })
    return parameters

def generate_alpha_16_ls(lookback_windows, contravariants, lags, up_thresholds, low_thresholds):
    parameters = []
    for lookback_window in lookback_windows:
        for up_threshold in up_thresholds:
            for low_threshold in low_thresholds:
                for lag in lags:
                    for contravariant in contravariants:
                        if lag <= lookback_window/4 and lag >= 2:
                            parameters.append({
                                'lookback_window':lookback_window,
                                'lag' : lag,
                                'low_threshold': low_threshold,
                                'up_threshold': up_threshold,
                                'contravariant':contravariant
                            })
    return parameters

def generate_alpha_6_ls(lookback_windows, vol_thresholds, contravariants, lags, display):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for vol_threshold in vol_thresholds:
                for lag in lags:
                    if lag <= int(lookback_window/2):
                        parameters.append({
                            'lookback_window':lookback_window,
                            'contravariant':contravariant,
                            'vol_threshold':vol_threshold,
                            'lag':lag,
                            'display' : display
                        })
    return parameters

def generate_volume_weighted_high_low_vol_lo(lookback_windows, vol_thresholds, up_trend_thresholds, contravariants, lags, display):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for vol_threshold in vol_thresholds:
                for up_trend_threshold in up_trend_thresholds:
                    for lag in lags:
                        if lag <= int(lookback_window/2):
                            parameters.append({
                                'lookback_window':lookback_window,
                                'contravariant':contravariant,
                                'vol_threshold':vol_threshold,
                                'up_trend_threshold':up_trend_threshold,
                                'lag':lag,
                                'display' : display
                            })
    return parameters

def generate_volume_weighted_high_low_vol_ls(lookback_windows, vol_thresholds, up_trend_thresholds,  low_trend_thresholds, contravariants, lags, display):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for vol_threshold in vol_thresholds:
                for up_trend_threshold in up_trend_thresholds:
                    for low_trend_threshold in low_trend_thresholds:
                        for lag in lags:
                            if lag <= int(lookback_window/2):
                                parameters.append({
                                    'lookback_window':lookback_window,
                                    'contravariant':contravariant,
                                    'vol_threshold':vol_threshold,
                                    'up_trend_threshold':up_trend_threshold,
                                    'low_trend_threshold': low_trend_threshold,
                                    'lag':lag,
                                    'display' : display
                                })
    return parameters

def generate_lo_slope_ma(lookback_windows, pente_windows, contravariants,):
    parameters = []
    for lookback_window in lookback_windows:
        for pente_window in pente_windows:
            for contravariant in contravariants:
                if pente_window <= lookback_window:
                    parameters.append({
                        'lookback_window':lookback_window,
                        'pente_window':pente_window,
                        'contravariant':contravariant
                    })
    return parameters

def generate_slope_lo_induced(lookback_windows, pente_windows, contravariants, lags, slope_columns, up_trend_thresholds, display):
    parameters = []
    for slope_column in slope_columns:
        for lookback_window in lookback_windows:
            for pente_window in pente_windows:
                for contravariant in contravariants:
                    for lag in lags:
                        for up_trend_threshold in up_trend_thresholds:
                            if pente_window < int(lookback_window/lag):
                                if lag < int(lookback_window/2):
                                    parameters.append({
                                        'lookback_window':lookback_window,
                                        'pente_window':pente_window,
                                        'contravariant':contravariant,
                                        'up_trend_threshold': up_trend_threshold,
                                        'lag':lag,
                                        'slope_column': slope_column,
                                        'display' : display
                                    })
    return parameters

def generate_slope_lo_to_ls(lookback_windows, pente_windows, pente_windows_bis, contravariants, lags, slope_columns, up_trend_thresholds, up_trend_thresholds_bis, display):
    parameters = []
    for slope_column in slope_columns:
        for lookback_window in lookback_windows:
            for pente_window in pente_windows:
                for pente_window_bis in pente_windows_bis:
                    for contravariant in contravariants:
                        for lag in lags:
                            for up_trend_threshold_bis in up_trend_thresholds_bis:
                                for up_trend_threshold in up_trend_thresholds:
                                    if pente_window < int(lookback_window/lag):
                                        if lag < int(lookback_window/2):
                                            parameters.append({
                                                'lookback_window':lookback_window,
                                                'pente_window':pente_window,
                                                'pente_window_bis':pente_window_bis,
                                                'contravariant':contravariant,
                                                'up_trend_threshold_bis': up_trend_threshold_bis,
                                                'up_trend_threshold': up_trend_threshold,
                                                'lag':lag,
                                                'slope_column': slope_column,
                                                'display' : display
                                            })
    return parameters

def generate_slope_ls_induced(lookback_windows, pente_windows, contravariants, lags, slope_columns, low_trend_thresholds, up_trend_thresholds, display):
    parameters = []
    for slope_column in slope_columns:
        for lookback_window in lookback_windows:
            for pente_window in pente_windows:
                for contravariant in contravariants:
                    for lag in lags:
                        for low_trend_threshold in low_trend_thresholds:
                            for up_trend_threshold in up_trend_thresholds:
                                if pente_window < int(lookback_window/lag):
                                    if lag < int(lookback_window/2):
                                        parameters.append({
                                            'lookback_window':lookback_window,
                                            'pente_window':pente_window,
                                            'contravariant':contravariant,
                                            'low_trend_threshold': low_trend_threshold,
                                            'up_trend_threshold': up_trend_threshold,
                                            'lag':lag,
                                            'slope_column': slope_column,
                                            'display' : display
                                        })
    return parameters

def generate_trending_rank_ls_induced(lookback_windows, contravariants, lags, slope_columns, up_trend_thresholds,  low_trend_thresholds, display):
    parameters = []
    for slope_column in slope_columns:
        for lookback_window in lookback_windows:
            for contravariant in contravariants:
                for lag in lags:
                    for up_trend_threshold in up_trend_thresholds:
                        for low_trend_threshold in low_trend_thresholds:
                            if lag <= int(lookback_window/2):
                                parameters.append({
                                    'lookback_window':lookback_window,
                                    'contravariant':contravariant,
                                    'up_trend_threshold': up_trend_threshold,
                                    'low_trend_threshold': low_trend_threshold,
                                    'lag':lag,
                                    'slope_column': slope_column,
                                    'display' : display
                                })
    return parameters

def generate_trending_rank_lo_induced(lookback_windows, contravariants, lags, slope_columns, up_trend_thresholds, display):
    parameters = []
    for slope_column in slope_columns:
        for lookback_window in lookback_windows:
            for contravariant in contravariants:
                for lag in lags:
                    for up_trend_threshold in up_trend_thresholds:
                        if lag <= int(lookback_window/2):
                            parameters.append({
                                'lookback_window':lookback_window,
                                'contravariant':contravariant,
                                'up_trend_threshold': up_trend_threshold,
                                'lag':lag,
                                'slope_column': slope_column,
                                'display' : display
                            })
    return parameters

def generate_slope_induced_cont(lookback_windows, contravariants, lags, slope_columns, display):
    parameters = []
    for slope_column in slope_columns:
        for lookback_window in lookback_windows:
            for contravariant in contravariants:
                for lag in lags:

                    if lag <= int(lookback_window/2):
                        parameters.append({
                            'lookback_window':lookback_window,
                            'contravariant':contravariant,
                            'lag':lag,
                            'slope_column': slope_column,
                            'display' : display
                        })
    return parameters


def generate_volume_weighted_high_low_vol_cont(lookback_windows, contravariants, lags, display):
    parameters = []
    for lookback_window in lookback_windows:
        for contravariant in contravariants:
            for lag in lags:
                if lag <= int(lookback_window/2):
                    parameters.append({
                        'lookback_window':lookback_window,
                        'contravariant':contravariant,
                        'lag':lag,
                        'display' : display
                    })
    return parameters


#!/usr/bin/env python
# coding: utf-8

from abc import ABC, abstractmethod

import pandas as pd

import numpy as np
from napoleontoolbox.file_saver import dropbox_file_saver
from napoleontoolbox.utility import weights
from napoleontoolbox.rebalancing import allocation


class AbstractRunner(ABC):
    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', returns_pkl_file_suffix='_returns.pkl', local_root_directory='../data/', user = 'napoleon', n_start = 252, transaction_costs = None, crypto_lo = False):
        super().__init__()
        self.supervision_npy_file_suffix = supervision_npy_file_suffix
        self.macro_supervision_npy_file_suffix = macro_supervision_npy_file_suffix
        self.features_saving_suffix = features_saving_suffix
        self.features_names_saving_suffix=features_names_saving_suffix
        self.returns_pkl_file_suffix=returns_pkl_file_suffix


        self.local_root_directory=local_root_directory
        self.user=user
        self.n_start = n_start
        self.dropbox_backup = dropbox_backup
        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
        self.save_model = save_model
        self.running_date = running_date
        self.starting_date = starting_date
        self.maximize_perf = True
        self.minimize_drawdown = True
        self.equaly_weighted = True
        self.erc_weighted = True
        self.ivp_allocation = True
        self.mvp_allocation = True
        self.hrp_allocation = True
        self.hrc_allocation = True
        self.mdp_allocation = True
        self.transaction_costs = None
        self.cypto_lo = crypto_lo
        if self.cypto_lo:
            self.returns_pkl_file_name =  'crypto_lo_' + self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
        else :
            self.returns_pkl_file_name = self.running_date.strftime('%d_%b_%Y') + self.returns_pkl_file_suffix
    @abstractmethod
    def runTrial(self,saver, seed,  n, s, low_bound, up_bound):
        pass


class SimpleEnsemblingRunner(AbstractRunner):
    def runTrial(self, saver, seed,  n, s, low_bound, up_bound):
        try:
            self.wrappedRunTrial(saver, seed, n, s, low_bound, up_bound)
        except Exception as e:
            print('################# : To investigate and relaunch')
            print('################# : To investigate and relaunch')
            print('################# : To investigate and relaunch')
            print(e)

    def wrappedRunTrial(self, saver, seed,  n, s, low_bound, up_bound):
        meArg = (seed, n, s, low_bound, up_bound)
        meArgList = list(meArg)
        meArgList = [str(it) for it in meArgList]
        savingKey = ''.join(meArgList)
        savingKey = savingKey.replace('[', '')
        savingKey = savingKey.replace(']', '')
        savingKey = savingKey.replace(',', '')
        savingKey = savingKey.replace(' ', '')
        savingKey = savingKey.replace('.', '')
        savingKey = 'T_' + savingKey

        print('Launching computation with parameters : '+savingKey)

        df = pd.read_pickle(self.local_root_directory + self.returns_pkl_file_name)

        print(df.columns)
        df['Date'] = pd.to_datetime(df['Date'])
        df = df.set_index('Date')
        df = df.fillna(method='ffill')
        df = df[df.index >= self.starting_date]
        print('max before filtering ' + str(max(df.index)))
        df = df[df.index <= self.running_date]

        print('max after filtering ' + str(max(df.index)))
        ### getting the transaction costs
        df_res = df.copy()
        df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
        if self.transaction_costs is None:
            print('no transaction costs given')
            costs_parameters = np.zeros(len(df_ret.columns))
        else:
            def get_fees_rate(asset, transaction_costs_dic):
                try:
                    fees = transaction_costs_dic[asset]
                except KeyError as e:
                    print(e)
                    return 0.
                return fees
            costs_parameters = np.fromiter(map(lambda x: get_fees_rate(x, self.transaction_costs), df_ret.columns),
                                           dtype=np.float64)        ### getting the transaction costs

        check_maxperf_existence = saver.checkRunExistence('mp_' + savingKey)
        if self.maximize_perf and not check_maxperf_existence:
            print('maximizing perf')

            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.maxPerf,
                df_res,
                n=n,
                s=s,
                normalize=True,
                low_bound=low_bound,
                up_bound=up_bound,
                filtering_threshold=0.9
            )

            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('mp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)




        check_drawdown_existence = saver.checkRunExistence('dd_'+savingKey)
        if self.minimize_drawdown and not check_drawdown_existence:
            print('minimizing drawdown')
            df_res = df.copy()
            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.minDrawdown,
                df_res,
                n=n,
                s=s,
                normalize=True,
                low_bound=low_bound,
                up_bound=up_bound,
                filtering_threshold=0.9
            )
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('dd_'+savingKey, portfolio, weight_normalized,last_predicted_weights)

        check_ew_existence = saver.checkRunExistence('ew_'+savingKey)
        if self.equaly_weighted and not check_ew_existence:
            print('equally weighted')
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('ew_'+savingKey, portfolio, weight_normalized,last_predicted_weights)


        check_erc_existence = saver.checkRunExistence('erc_'+savingKey)
        if self.erc_weighted and not check_erc_existence:
            print('computing erc weights')
            df_res = df.copy()
            # Compute rolling weights
            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.ERC,
                df_res,
                n=n,
                s=s,
                ret=False,
                low_bound=low_bound,
                up_bound=up_bound,
                filtering_threshold=0.9
            )
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('erc_'+savingKey, portfolio, weight_normalized,last_predicted_weights)


        check_ivp_existence = saver.checkRunExistence('ivp_'+savingKey)
        if self.ivp_allocation and not check_ivp_existence:
            print('computing ivp weights')
            # Compute rolling weights
            df_res = df.copy()
            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.IVP,
                df_res,
                n=n,
                s=s,
                normalize=True,
                low_bound=low_bound,
                up_bound=up_bound,
                filtering_threshold=0.9
            )
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('ivp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)



        check_mpv_existence = saver.checkRunExistence('mvp_'+savingKey)
        if self.mvp_allocation and not check_mpv_existence:
            print('computing mvp weights')
            df_res = df.copy()
            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.MVP_uc,
                df_res,
                n=n,
                s=s,
                low_bound=low_bound,
                up_bound=up_bound,
                ret=False,
                filtering_threshold=0.9
            )
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('mvp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)


        check_hrp_existence = saver.checkRunExistence('hrp_'+savingKey)
        if self.hrp_allocation and not check_hrp_existence:
            print('computing hrp weights')
            df_res = df.copy()
            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.HRP,
                df_res,
                n=n,
                s=s,
                ret=True,
                drift=True,
                method='centroid',
                metric='mse',
                up_bound=up_bound,
                low_bound=low_bound,
                filtering_threshold=0.9
            )
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('hrp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)


        check_hrc_existence = saver.checkRunExistence('hrc_'+savingKey)
        if self.hrc_allocation and not check_hrc_existence:
            print('computing hrc weights')
            df_res = df.copy()
            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.HRC,
                df_res,
                n=n,
                s=s,
                up_bound=up_bound,
                low_bound=low_bound,
            )
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('hrc_'+savingKey, portfolio, weight_normalized,last_predicted_weights)

        check_mdp_existence = saver.checkRunExistence('mdp_'+savingKey)
        if self.mdp_allocation and not check_mdp_existence:
            print('computing mdp weights')
            # Compute rolling weights
            df_res = df.copy()
            portfolio, weight_mat = allocation.rolling_allocation(
                allocation.MDP,
                df_res,
                n=n,
                s=s,
                ret=False,
                drift=True,
                up_bound=up_bound,
                low_bound=low_bound,
                filtering_threshold=0.9
            )
            df_ret = df_res.fillna(method='bfill').pct_change().fillna(0).copy()
            weight_normalized, last_predicted_weights = weights.weights_shift(weight_mat=weight_mat)
            #portfolio = np.cumprod(np.prod(df_ret * weight_normalized.values + 1, axis=1))
            # # Compute portfolio perf
            transaction_costs_matrix = abs(weight_normalized - weight_normalized.shift(1)) *costs_parameters
            adjusted_returns = df_ret * weight_normalized.values - transaction_costs_matrix
            portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
            saver.saveResults('mdp_'+savingKey, portfolio, weight_normalized,last_predicted_weights)



__author__ = "hugo.inzirillo"

from dataclasses import dataclass
from typing import List, Dict

from pandas import Timestamp

from napoleontoolbox.models.product import Product
from napoleontoolbox.models.quantity import Quantity
from napoleontoolbox.models.signal import SignalComputation


@dataclass
class PortfolioPk:
    amount: float
    leverage: float
    composition: Dict[str, float]
    start_date: Timestamp
    end_date: Timestamp

    def __hash__(self):
        return hash((self.amount, self.leverage, self.composition, self.start_date, self.end_date))

    def __str__(self):
        return "PortfolioPk{amount=" + str(
            self.amount) + ", start_date=" + self.start_date + ", end_date=" + self.end_date + ", leverage=" + str(
            self.leverage) + '}'


@dataclass
class Portfolio:
    pk: PortfolioPk
    products: List[Product] = None
    positions: List[SignalComputation] = None
    quantities: List[Quantity] = None

    def __hash__(self):
        return hash((self.pk))

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return self.pk.__str__()

    def __eq__(self, other):
        return self.pk == other.pk if isinstance(other, self.__class__) else False


@dataclass
class PortfolioView:
    amount: float = None
    positions: List[SignalComputation] = None
    quantities: List[Quantity] = None

    def from_dict(self, dict_object: dict):
        if isinstance(dict_object, dict):
            self.__dict__ = {key: value for key, value in dict_object.items()
                             if key in set(dict_object.keys()).intersection(set(self.__dict__.keys()))}
            return self

    def __str__(self):
        return "Portfolio{amount=" + str(
            self.amount) + '}'


if __name__ == '__main__':
    pk = PortfolioPk(1000, 1.5, dict(STRAT_BTC_ETH_USD_LO_H_1=0.5, STRAT_BTC_ETH_USD_D_1=0.5), Timestamp.now(),
                     Timestamp.now())
    ptf = Portfolio(pk=pk)

    end = True



__author__ = "hugo.inzirillo"

from dataclasses import dataclass


@dataclass(init=True)
class ArchTestResult:
    lm_stat: float
    lm_pval: float
    f_stat: float
    f_pval: float

    @property
    def arch_effect(self):
        return self.f_pval <= 0.05
